import React from 'react';
import { ButtonClickHandler } from '../Button';
declare type BaseElement = HTMLDivElement;
declare type BaseProps = React.HTMLAttributes<BaseElement>;
export interface IStepContent extends BaseProps {
    readonly label: string;
    readonly content: React.ReactNode;
    readonly hasErrors?: boolean;
    readonly disableNext?: boolean;
}
export interface IStepperAction {
    /**
     * The label for the stepper button (previous, next or complete action).
     */
    readonly label: string;
    /**
     * The click handler for the stepper button (previous, next or complete action).
     */
    readonly onClick?: ButtonClickHandler;
}
export interface IStepperProps extends BaseProps {
    /**
     * `class` to be passed to the component.
     */
    readonly className?: BaseProps['className'];
    /**
     * The steps passed to the stepper.
     */
    readonly steps: ReadonlyArray<IStepContent>;
    /**
     * The function that run on action button click in the last stepper step.
     */
    readonly completeAction: IStepperAction;
    /**
     * The function that runs on previous button click.
     */
    readonly prevAction: IStepperAction;
    /**
     * The function that runs on next button click.
     */
    readonly nextAction: IStepperAction;
    /**
     * The function that runs on reset button click.
     */
    readonly resetAction: IStepperAction;
}
export declare const Stepper: React.FC<IStepperProps>;
export {};
