"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Table = exports.useGridTemplateColumns = exports.TableContext = exports.WidthActionType = exports.TABLE_DIMENSIONS = void 0;

var _react = _interopRequireWildcard(require("react"));

var _styledComponents = _interopRequireDefault(require("styled-components"));

var _designparams = require("../designparams");

var _ColumnResizerRow = require("./ColumnResizerRow");

var _reactHooksShareable = require("react-hooks-shareable");

var _GlobalScrollbarStyle = require("../Global/GlobalScrollbarStyle");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var PADDING_LEFT = 24;
var TABLE_DIMENSIONS = {
  DEFAULT_MIN_COLUMN_WIDTH: 64,
  PADDING_LEFT,
  PADDING: PADDING_LEFT + _GlobalScrollbarStyle.SCROLLBAR_DIMENSION,
  // total padding (sum of left and right(scrollbar))
  SELECT_WIDTH: 48,
  // holds the checkbox
  MENU_WIDTH: 32 + (20 - _GlobalScrollbarStyle.SCROLLBAR_DIMENSION),
  // holds the menu (24px box that holds menu button icon should have 24px space between itself and Table edge)
  // TODO: this is componentSize, but that one is a string and contains 'px'
  ROW_HEIGHT: 56
};
/*******************************************************************************
 *
 * Column width control.
 *
 * The columns of a table always have a fixed width,
 * which is set initially either by the user or by
 * detecting the rendered size of the table, and dividing
 * it by the available columns. When you specify your
 * own initial widths, they will be scaled to the table
 * size automatically (and when resizing).
 *
 * The array of widths is updated using a reducer.
 *
 ******************************************************************************/

exports.TABLE_DIMENSIONS = TABLE_DIMENSIONS;
var WidthActionType;
exports.WidthActionType = WidthActionType;

(function (WidthActionType) {
  WidthActionType[WidthActionType["SET_WIDTHS"] = 0] = "SET_WIDTHS";
  WidthActionType[WidthActionType["UPDATE_WIDTHS"] = 1] = "UPDATE_WIDTHS";
  WidthActionType[WidthActionType["RESET_WIDTHS"] = 2] = "RESET_WIDTHS";
  WidthActionType[WidthActionType["SET_TOTAL_WIDTH"] = 3] = "SET_TOTAL_WIDTH";
})(WidthActionType || (exports.WidthActionType = WidthActionType = {}));

var reduceWidths = (state, action) => {
  switch (action.type) {
    case WidthActionType.SET_WIDTHS:
      {
        // Sanitize columns to the minimum width
        var sanitizedWidths = action.widths.map(width => Math.max(state.minColumnWidth, width));
        var totalColumnWidths = sanitizedWidths.reduce((acc, cur) => acc + cur, 0);
        var normalizedWidths = sanitizedWidths.map(width => width / totalColumnWidths); // Scale all columns to fill the total width.

        return _objectSpread(_objectSpread({}, state), {}, {
          columnWidths: normalizedWidths.map(normalizedWidth => Math.max(state.minColumnWidth, normalizedWidth * state.totalWidth))
        });
      }

    case WidthActionType.UPDATE_WIDTHS:
      {
        var columnWidths = state.columnWidths.map((value, id) => {
          var width = action.widths[id];
          return width !== undefined ? Math.max(state.minColumnWidth, width) : value;
        });

        if (action.onWidthsChange !== undefined) {
          action.onWidthsChange(columnWidths);
        }

        return _objectSpread(_objectSpread({}, state), {}, {
          columnWidths
        });
      }

    case WidthActionType.RESET_WIDTHS:
      {
        var columnWidth = action.numberOfColumns !== 0 ? state.totalWidth / action.numberOfColumns : 0;
        return _objectSpread(_objectSpread({}, state), {}, {
          columnWidths: new Array(action.numberOfColumns).fill(Math.max(state.minColumnWidth, columnWidth))
        });
      }

    case WidthActionType.SET_TOTAL_WIDTH:
      {
        var totalWidth = action.value; // We always base the calculation on the initial widths if possible,
        // in order to keep the widths stable upon resize.

        var _columnWidths = state.initialWidths.length !== 0 ? state.initialWidths : state.columnWidths;

        var _totalColumnWidths = _columnWidths.reduce((acc, cur) => acc + cur, 0);

        var nColumns = _columnWidths.length;

        if (totalWidth === _totalColumnWidths || nColumns === 0) {
          return _objectSpread(_objectSpread({}, state), {}, {
            totalWidth
          });
        }

        var minWidth = nColumns * state.minColumnWidth;
        var availableWidth = totalWidth - minWidth;

        if (availableWidth <= 0) {
          // There is no available width, all columns get the minimum width
          return _objectSpread(_objectSpread({}, state), {}, {
            totalWidth,
            columnWidths: _columnWidths.map(() => state.minColumnWidth)
          });
        } // Distribute available width to all columns


        var _sanitizedWidths = _columnWidths.map(width => Math.max(state.minColumnWidth, width));

        var sanitizedTotalWidth = _sanitizedWidths.reduce((acc, cur) => acc + cur, 0);

        var extraColumnWidth = sanitizedTotalWidth - minWidth;

        if (extraColumnWidth <= 0) {
          // There is no extra width, all columns get the same new width
          return _objectSpread(_objectSpread({}, state), {}, {
            totalWidth,
            columnWidths: _columnWidths.map(() => totalWidth / nColumns)
          });
        }

        var _normalizedWidths = _sanitizedWidths.map(width => Math.max(width - state.minColumnWidth) / extraColumnWidth);

        return _objectSpread(_objectSpread({}, state), {}, {
          totalWidth,
          columnWidths: _normalizedWidths.map(normalizedWidth => state.minColumnWidth + normalizedWidth * availableWidth)
        });
      }

    default:
      {
        return state;
      }
  }
};
/*******************************************************************************
 *
 * TableContext
 *
 * Keeps track of table-wide properties which need to
 * be accessible from within each of the subcomponents.
 *
 ******************************************************************************/


var TableContext = /*#__PURE__*/(0, _react.createContext)({
  minColumnWidth: 0,
  columnWidths: [],
  dispatchWidthsAction: () => {
    /** */
  },
  selectWidth: 0,
  menuWidth: 0,
  hasMenu: false,
  onWidthsChange: () => {
    /** */
  }
});
/**
 * Transform the array of widths to an appropriate grid-template-columns
 * string, taking into account select/menu columns.
 *
 * Example:
 *  [34, 78]
 */

exports.TableContext = TableContext;

var useGridTemplateColumns = () => {
  var {
    columnWidths,
    selectWidth,
    menuWidth
  } = (0, _react.useContext)(TableContext); // Set up grid template columns from widths

  var gridTemplateColumnsStyle = (0, _react.useMemo)(() => {
    return {
      gridTemplateColumns: ["".concat(TABLE_DIMENSIONS.PADDING_LEFT, "px"), '[start]', "".concat(selectWidth, "px"), '[content-start]', ...columnWidths.map(columnWidth => "".concat(columnWidth, "px")), '[content-end]', "".concat(menuWidth, "px"), '[scroll]', "".concat(_GlobalScrollbarStyle.SCROLLBAR_DIMENSION, "px"), '[end]'].join(' ')
    };
  }, [columnWidths, menuWidth, selectWidth]);
  return gridTemplateColumnsStyle;
};
/*******************************************************************************
 *
 * Internal Table building blocks.
 *
 ******************************************************************************/

/**
 * The basic components of our grid.
 *
 * Note: grid-template-columns is set using a style attribute
 * to allow for dynamic change (without affecting the class name)
 */


exports.useGridTemplateColumns = useGridTemplateColumns;

var TableContainer = _styledComponents.default.div.withConfig({
  displayName: "Table__TableContainer",
  componentId: "sc-1cg7sno-0"
})(["overflow-x:auto;overflow-y:hidden;position:relative;pointer-events:", ";height:100%;width:100%;background-color:", ";"], (_ref) => {
  var {
    dragging
  } = _ref;
  return dragging ? 'none' : undefined;
}, (_ref2) => {
  var {
    theme
  } = _ref2;
  return theme.color.background00();
});
/**
 * TableHeaderGrid
 *
 * A grid containing the header cells.
 */


var TableHeaderContainer = _styledComponents.default.div.withConfig({
  displayName: "Table__TableHeaderContainer",
  componentId: "sc-1cg7sno-1"
})(["display:grid;width:100%;border-bottom:1px solid ", ";"], (_ref3) => {
  var {
    theme
  } = _ref3;
  return theme.color.element12();
});
/**
 * TableContentGrid
 *
 * A grid containing the content cells.
 */


var TableContentContainer = _styledComponents.default.div.attrs((_ref4) => {
  var {
    width,
    maxHeight
  } = _ref4;
  return {
    style: {
      width: "".concat(width, "px"),
      maxHeight: "".concat(maxHeight, "px")
    }
  };
}).withConfig({
  displayName: "Table__TableContentContainer",
  componentId: "sc-1cg7sno-2"
})(["display:grid;overflow-y:auto;overflow-x:hidden;padding-bottom:", ";position:relative;z-index:0;"], _designparams.spacing.medium);
/**
 * Table
 *
 * Defined by the number of columns.
 */


var EMPTY_ARRAY = [];

var Table = /*#__PURE__*/_react.default.memo((_ref5) => {
  var {
    initialWidths = EMPTY_ARRAY,
    resizableColumns = false,
    minColumnWidth = TABLE_DIMENSIONS.DEFAULT_MIN_COLUMN_WIDTH,
    onWidthsChange,
    maxHeight,
    onSelect,
    hasMenu = false,
    scrollKey,
    children
  } = _ref5,
      props = _objectWithoutProperties(_ref5, ["initialWidths", "resizableColumns", "minColumnWidth", "onWidthsChange", "maxHeight", "onSelect", "hasMenu", "scrollKey", "children"]);

  // Table width/height, including column widths
  var [{
    columnWidths
  }, dispatchWidthsAction] = (0, _react.useReducer)(reduceWidths, {
    totalWidth: initialWidths.reduce((sum, width) => sum + width, 0),
    columnWidths: initialWidths,
    initialWidths,
    minColumnWidth
  });
  var [tableHeight, setTableHeight] = (0, _react.useState)(); // Keep track of additional columns for select/menu.

  var selectWidth = (0, _react.useMemo)(() => onSelect !== undefined ? TABLE_DIMENSIONS.SELECT_WIDTH : 0, [onSelect]);
  var menuWidth = (0, _react.useMemo)(() => hasMenu ? TABLE_DIMENSIONS.MENU_WIDTH : 0, [hasMenu]); // Set up total width control

  var tableRef = (0, _react.useRef)(null);
  (0, _react.useLayoutEffect)(() => {
    var tableEl = tableRef.current;

    if (tableEl !== null) {
      var updateTableDimensions = () => {
        var {
          width,
          height
        } = tableEl.getBoundingClientRect();
        dispatchWidthsAction({
          type: WidthActionType.SET_TOTAL_WIDTH,
          value: width - selectWidth - menuWidth - TABLE_DIMENSIONS.PADDING
        });
        setTableHeight(height);
      };

      updateTableDimensions();
      var observer = new window.ResizeObserver(updateTableDimensions);
      observer.observe(tableEl);
      return () => observer.disconnect();
    }
  }, [columnWidths.length, selectWidth, menuWidth, setTableHeight]);
  var [dragging, setDragging] = (0, _react.useState)(false);
  /**
   * The content's width and (maximum) height.
   */

  var contentWidth = (0, _react.useMemo)(() => {
    return columnWidths.reduce((total, width) => total + width, 0) + selectWidth + menuWidth + TABLE_DIMENSIONS.PADDING;
  }, [columnWidths, selectWidth, menuWidth]);
  var contentHeight = maxHeight !== undefined ? maxHeight * TABLE_DIMENSIONS.ROW_HEIGHT : tableHeight !== undefined ? tableHeight - TABLE_DIMENSIONS.ROW_HEIGHT : undefined;
  var {
    header,
    rows
  } = (0, _react.useMemo)(() => {
    var [headerEl, ...rowsEl] = _react.default.Children.toArray(children);

    return {
      header: headerEl,
      rows: rowsEl
    };
  }, [children]);
  var tableContentRef = (0, _react.useRef)(null); // Scroll to top when scrollKey changes

  (0, _reactHooksShareable.useResetScroll)(tableContentRef, scrollKey);
  return /*#__PURE__*/_react.default.createElement(TableContainer, _extends({
    ref: tableRef,
    dragging: dragging
  }, props), /*#__PURE__*/_react.default.createElement(TableContext.Provider, {
    value: {
      minColumnWidth,
      columnWidths,
      dispatchWidthsAction,
      selectWidth,
      menuWidth,
      onSelect,
      hasMenu,
      onWidthsChange
    }
  }, /*#__PURE__*/_react.default.createElement(TableHeaderContainer, null, header), /*#__PURE__*/_react.default.createElement(TableContentContainer, {
    maxHeight: contentHeight,
    width: contentWidth,
    ref: tableContentRef
  }, tableHeight !== undefined ? rows : undefined), resizableColumns ? /*#__PURE__*/_react.default.createElement(_ColumnResizerRow.ColumnResizerRow, {
    setDragging: setDragging
  }) : null));
});

exports.Table = Table;